#include <Basis/Screen.hpp>
#include "Play.hpp"

#include <Graphics/Graphics.hpp>

#include <SDL.h>
#include <SDL_opengl.h>

#include <Basis/Log.hpp>

int main(int argc, char **argv) {
    if (argc < 4) {
        printf("Please pass in a hostname, a port, and a game name.\n");
        return 1;
    }

	if (SDL_Init(SDL_INIT_TIMER) != 0) {
		LOG_ERROR("Could not initialize sdl: " << SDL_GetError());
		return 1;
	}

	//ask for a specific screen size:
	Graphics::screen.x = 800;
	Graphics::screen.y = 600;
	Graphics::fullscreen = false;

	if (!Graphics::init()) {
		LOG_ERROR("Could not initialize graphics, not continuing.");
		exit(1);
	}

	//Some basic GL state:
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
    gluOrtho2D(0.0, Graphics::screen.x, 0.0, Graphics::screen.y);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_LINE_SMOOTH);

	{
		Play play(argv[1], argv[2], argv[3]);
		//This will return when someone does quit_screen(play):
		start_screen(&play);
	}

	Graphics::deinit();

	SDL_Quit();

	return 0;
}
