#ifndef CLIENT_HPP
#define CLIENT_HPP

#include <ev.h>
#include <vector>
#include <string>
#include <list>

using namespace std;

#define rand_max(x) ((int)((rand() / (float)RAND_MAX) * (x)))

#define BOARD_WIDTH 20
#define BOARD_HEIGHT 10
#define NUM_MINES 30
#define MINE (-1)

class Game;

class Client {
private:
    void reveal_squares(Game *g, int x, int y);
public:
	Client(int sock);
	~Client();

	int sock;
	ev_io read_watcher;
	ev_io write_watcher;
	vector< uint8_t > read_buffer;
	vector< uint8_t > write_buffer;
	bool suicide; // Close connection?
	string name; // Name of the game we are joining

    void send_square(Game g, int x, int y);
	void send_message(string message, char type);
	void send_error(string message);
	void write_data(const void *data, unsigned int bytes);
	bool process_read();
};

class Game {
public:
    list< Client * > players;
    bool new_game, game_over;
    int board[BOARD_WIDTH][BOARD_HEIGHT];
    int revealed[BOARD_WIDTH][BOARD_HEIGHT];

    Game() : new_game(true), game_over(false) {};
    void increment_surrounding(int x, int y);
    void generate_board();
};

#endif //CLIENT_HPP
