#include "Client.hpp"

#include <ev.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <memory.h>

#include <iostream>
#include <fstream>
#include <cstdlib>

using std::cout;
using std::endl;
using std::ifstream;
using std::getline;

void sigint(int) {
	cout << "Shutting down." << endl;
	exit(0);
}

void sock_cb(struct ev_loop *loop, ev_io *w, int revents) {
	//accept some connections.
	int fd = accept(w->fd, NULL, NULL);
	if (fd == -1) {
		perror("Accept error on sock");
		exit(1);
	} else {
		cout << "Creating new client." << endl;
		new Client(fd);
	}
}

int main() {
	signal(SIGINT, sigint);
	signal(SIGPIPE, SIG_IGN); 
	unsigned int port = 13126;

	// Create socket
	int sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock == -1) {
		perror("Open socket.");
		exit(1);
	}

	// Enable reuse
    int one = 1;
    if (0 != setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,&one,sizeof(int))) {
        perror("Error setting SO_REUSEADDR.");
        exit(1);
    }

	// Bind socket to port
    struct sockaddr_in serv_addr;
    memset((char *)&serv_addr, 0, sizeof(sockaddr_in));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = INADDR_ANY;
    serv_addr.sin_port = htons(port);
    if (0 != bind(sock,(struct sockaddr *)&serv_addr,sizeof(sockaddr_in))) {
        perror("Binding sock");
        close(sock);
        exit(1);
    }
    if (0 != listen(sock, 5)) {
        perror("Listening on sock");
        close(sock);
        exit(1);
    }

	// Kick off the event loop
	struct ev_loop *loop = ev_default_loop(EVFLAG_AUTO);
	ev_io sock_watcher;
	ev_io_init(&sock_watcher, sock_cb, sock, EV_READ);
	ev_io_start(loop, &sock_watcher);
	cout << "Listening on " << port << "..." << endl;
	ev_loop(loop, 0);

	// Probably never returns...
	close(sock);
	exit(0);
}
